#!/bin/bash
# Sets up a WebLogic domain for PPS.
#
# The JAVA_HOME and MW_HOME environment variables must be set before running this script.

# Fail immediately on any error
set -e

# *** Script configuration ***

# The path to the installation resources. (Assumed to be the path to this script.)
install_dir=$(dirname "$0")

# The path of the WebLogic domain to create.
domain_dir=$HOME/weblogic/PPS_domain

# The password for the 'weblogic' domain user.
password_domain="welcome1"

# Property files for datasources to create
datasource_files="$install_dir/properties/NationalEPL-ds.properties
    $install_dir/properties/FDBDIF-ds.properties
    $install_dir/properties/FSS-ds.properties
    $install_dir/properties/vha-stddata-pool-ds.properties"

# *** Check prerequisites ***
if grep -l "FILL ME IN" $datasource_files; then
    echo "Error: You have not yet updated the above datasource properties files. Please update"
    echo "them and rerun this script."
    exit 1
fi

# *** Now on to business ***

echo unpacking and creating domain...
"$MW_HOME/oracle_common/common/bin/unpack.sh" \
    -template="$install_dir/domain_template.jar" -domain="$domain_dir" \
    -user_name=weblogic -password="$password_domain"

# Also configure log4j in the domain.
"$install_dir/configureLog4j2.sh" "$domain_dir"

# *** Start the domain (in the background) ***
echo "Starting WebLogic domain..."
"$domain_dir/startWebLogic.sh" >"$HOME/weblogic.out" 2>&1 &

echo "Giving WebLogic 20 seconds to start up before trying to configure the domain..."
sleep 20

# *** Create datasources ***

# Setup WLST environment. Note: this script changes the working directory.
source $domain_dir/bin/setDomainEnv.sh

for propsFile in $datasource_files; do
    echo "Creating datasource using $propsFile..."
    java weblogic.WLST "$install_dir/create_datasource.py" -p "$propsFile"
done

echo "Stopping WebLogic after domain configuration. Enjoy!"
"$domain_dir/bin/stopWebLogic.sh"

